@@ -28,16 +28,4 @@ else:
     from deep_utils.utils.utils.dictnamedtuple_37 import dictnamedtuple
 
 
-def easy_argparse(*args):
-    parser = argparse.ArgumentParser()
 
-    for arg in args:
-        tp = type(arg)
-        if tp == dict:
-            name = arg.pop('name')
-            parser.add_argument(name, **arg)
-        else:
-            raise TypeError(
-                f"easy_argparse just supports dictionaries"
-            )
-    return parser.parse_args()
